// Ansteuerung Multiplex-Platine fuer NXT
// Alle Eingaenge werden einfach staendig abgefragt und
// anschliessend logisch verknuepft.
// Optionen zur Energieeinsparung (sleep-Modi mit Aufwecken
// ueber Pin-Change-Interrupt) wurden nicht realisiert.

// Der Ausgang ist hochohmig oder gegen Masse gezogen
// Die LED hat die gleiche Logik

// Eingang der DIL-Schalter (Inverse Logik):
// PB0 - PB5, PC0, PC1

// Eingang des DIL-Schalters UND oder ODER:
// PC4 (SW11)

// Eingang des DIL-Schalters SW12 wird nicht benutzt.

// Sensor-Eingnge:
// PD0 - PD7



#include <avr/io.h>

#define PORT_AUS PORTC
#define PIN_AUS PC3
#define PIN_LED PC2
#define SW11 PC4
#define SW12 PC5



// Globale Variablen
 volatile unsigned char invlogik=0;
 unsigned char logik=0;
 volatile unsigned char taster=0;


inline void switch_on(void)
{
 DDRC |= (1 << PIN_AUS) | (1 << PIN_LED);
}

inline void switch_off(void)
{
 DDRC &= ~((1 << PIN_AUS) | (1 << PIN_LED));
}


inline void abfrage_dipschalter(void)
{
 // Pullups aktivieren
  PORTB = 0xFF;
  PORTC |= ((1 << PC0) | (1 << PC1) );

  // Pins der Dipschalter einlesen
  invlogik = (PINB & 0b00111111) | ((PINC & 0b00000011) << 6 ); 
 
  // Pullups deaktivieren
  PORTB = 0x00;
  PORTC &= ~((1 << PC0) | (1 << PC1) );
}

inline void abfrage_taster(void)
{
 // Pullups aktivieren
  PORTD = 0xFF;

 // Pins der Taster einlesen
 taster= ~PIND;

 // Pullups deaktivieren
 PORTD = 0x00;
}





int main(void)
{
 
 // Port B, PC0, PC1 sind Eingang 
 DDRB = 0x00;
 DDRC &= ~((1 << PC0) | (1 << PC1) );

 // Port D ist Eingang
 DDRD  = 0x00;

 // PC4 und PC5 sind Eingang mit Pullup
 DDRC &= ~((1 << PC4) | (1 << PC5));
 PORTC |= ((1 << PC4) | (1 << PC5));

 unsigned char invlogik=0;
 unsigned char logik=0;
 while(1)
 {
   abfrage_dipschalter();
   abfrage_taster();

      

   logik = taster ^ ~invlogik;
     if((PINC & (1 << SW11))) // ODER-Modus
     {
       if (logik==0)
       {
         switch_off();
       }
       else
       {
         switch_on();
       }
     }
     else // UND-Modus
     {
       if (logik == 0xFF)
       {
         switch_on();
       }
       else
       {
         switch_off();
       }
     }
  }
}
